@@ -28,6 +28,7 @@ import cartopy.feature
 from cartopy.mpl.gridliner import LONGITUDE_FORMATTER, LATITUDE_FORMATTER
 import pandas as pd
 from adjustText import adjust_text as adj_txt
+import os
 
 ############################################################################
 # MISCELLANEOUS FUNCTIONS
@@ -87,6 +88,80 @@ def cal_dist_dir_on_sphere(longitude, latitude):
     
     return speed, heading
 
+def cart2pol(u,v,ctype='math'):
+    '''
+    Converts cartesian velocity (u,v) to polar velocity (angle,speed), 
+    using either 
+    1) mathematical
+    2) oceanographical, or
+    3) meteorological 
+    definition.
+
+    Parameters
+    ----------
+    u : numeric, or array-like
+        u-Component of velocity.
+    v : numeric, or array-like
+        v-Component of velocity.
+    ctype : string, optional
+        Type of definitition, 'math', 'ocean' or 'meteo'. The default is 'math'.
+
+    Returns
+    -------
+    angle : numeric, or array-like
+        Angle of polar velocity.
+    speed : numeric, or array-like
+        Speed of polar velocity.
+
+    '''
+    speed = np.sqrt(u**2 + v**2)
+    if ctype == 'math':
+        angle = 180/np.pi* np.arctan2(v,u)
+    if ctype in ['meteo','ocean']:
+        angle = 180 / np.pi * np.arctan2(u,v)         
+        if ctype == 'meteo':
+            angle = (angle+180)%360
+        
+    return angle,speed
+
+def pol2cart(angle,speed,ctype='math'):
+    '''
+    Converts polar velocity (angle,speed) to cartesian velocity (u,v), 
+    using either 
+    1) mathematical
+    2) oceanographical, or
+    3) meteorological 
+    definition.
+
+    Parameters
+    ----------
+    angle : numeric, or array-like
+        Angle of polar velocity.
+    speed : numeric, or array-like
+        Speed of polar velocity.
+    ctype : string, optional
+        Type of definitition, 'math', 'ocean' or 'meteo'. The default is 'math'.
+
+    Returns
+    -------
+    u : numeric, or array-like
+        u-Component of velocity.
+    v : numeric, or array-like
+        v-Component of velocity.
+
+    '''
+    if ctype == 'math':
+        u = speed * np.cos(angle*np.pi/180.)
+        v = speed * np.sin(angle*np.pi/180.)
+    elif ctype == 'meteo':
+        u = -speed * np.sin(angle*np.pi/180.)
+        v = -speed * np.cos(angle*np.pi/180.)
+    elif ctype == 'ocean':
+        u = speed * np.sin(angle*np.pi/180.)
+        v = speed * np.cos(angle*np.pi/180.)
+    
+    return u,v
+        
 def create_latlon_text(lat,lon):
     '''
     Creates two strings which contain a text for latitude and longitude
@@ -122,7 +197,7 @@ def create_latlon_text(lat,lon):
     
     return latstring,lonstring
 
-def CTD_to_grid(CTD,stations=None,interp_opt= 1,x_type='distance'):
+def CTD_to_grid(CTD,stations=None,interp_opt= 1,x_type='distance',z_fine=False):
     '''
     This function accepts a CTD dict of dicts, finds out the maximum 
     length of the depth vectors for the given stations, and fills all
@@ -141,7 +216,7 @@ def CTD_to_grid(CTD,stations=None,interp_opt= 1,x_type='distance'):
                      2: linear interpolation, coarse grid. The default is 1.
     x_type : str, optional
         whether X is 'time' or 'distance'. The default is 'distance'.
-
+    z_fine: Whether to use a fine z grid. If True, will be 10 cm, otherwise 1 m
     Returns
     -------
     fCTD : dict
@@ -164,7 +239,10 @@ def CTD_to_grid(CTD,stations=None,interp_opt= 1,x_type='distance'):
     # construct the Z-vector from the max and min depth of the given stations
     maxdepth = np.nanmax([np.nanmax(-CTD[i]['z']) for i in stations])
     mindepth = np.nanmin([np.nanmin(-CTD[i]['z']) for i in stations])
-    Z = np.linspace(mindepth,maxdepth,int(maxdepth-mindepth)+1)
+    if z_fine:
+        Z = np.linspace(mindepth,maxdepth,int((maxdepth-mindepth)*10)+1)
+    else:
+        Z = np.linspace(mindepth,maxdepth,int(maxdepth-mindepth)+1)
     
     # construct the X-vector, either distance or time
     if x_type == 'distance':
@@ -462,7 +540,7 @@ def read_CTD(inpath,cruise_name='cruise',outpath=None,stations=None,
                  'CNDC2':'C2','PSAL':'S','PSAL2':'S2','oxygen_ml_L':'OX',
                  'flC':'flC','par':'PAR','altM':'Z','timeJ':'time',
                  'timeK':'time','timeS':'elapsedtime','longitude':'lon',
-                 'latitude':'lat'}
+                 'latitude':'lat','TEMP':'T','oxsolML/L':'OX','flSP':'flC'}
     # get all CTD station files in inpath
     files = glob.glob(inpath+'*.cnv')
     #If stations are provided, select the ones that exist
@@ -498,15 +576,15 @@ def read_CTD(inpath,cruise_name='cruise',outpath=None,stations=None,
         p['z'] = gsw.z_from_p(p['P'],p['LAT'])
         p['BottomDepth'] = np.round(np.nanmax(np.abs(p['z']))+8)
         p['C'][p['C']<1] = np.nan
-        p['C'] = salt_corr[0] * p['C'] + salt_corr[1] # apply correction
+        p['C'] = salt_corr[0] * p['C']*10 + salt_corr[1] # apply correction
         p['T'][p['T']<-2] = np.nan
-        p['S'] = gsw.SP_from_C(p['C']*10,p['T'],p['P'])
+        p['S'] = gsw.SP_from_C(p['C'],p['T'],p['P'])
         p['S'][p['S']<20] = np.nan
         p['C'][p['S']<20] = np.nan
         p['SA'] = gsw.SA_from_SP(p['S'],p['P'],p['LON'],p['LAT'])
         p['CT'] = gsw.CT_from_t(p['SA'],p['T'],p['P'])
         p['SIGTH'] = gsw.sigma0(p['SA'],p['CT'])
-        p['st'] = int(p['filename'].split('.')[0][-4::])
+        p['st'] = int(p['filename'].split('.')[0].split('_')[0][-4::])
         if 'OX' in p:
             p['OX'] = oxy_corr[0] * p['OX'] + oxy_corr[1]
         CTD_dict[p['st']]= p
@@ -648,7 +726,76 @@ def read_mini_CTD(file,corr=(1,0),lon=0,lat=60.,station_name = 'miniCTD'):
         del p['Date'], p['Time']
 
     return p    
+
+def read_MSS(files,excel_file=None):
+    '''
+    Parameters
+    ----------
+    file : str
+        Full path to the .mat file.
+
+    Returns
+    -------
+    None.
+
+    '''  
+    # first, handle the excel file
+    if excel_file is not None:
+        exc = pd.read_excel(excel_file)
+        exc.columns = np.arange(len(exc.columns))
+        st = exc[[a for a in exc.columns 
+                    if 'Station name' in exc[a].to_numpy()][0]]
+        lat_deg = exc[[a for a in exc.columns 
+                    if 'Latitude/ N' in exc[a].to_numpy()][0]]
+        lat_min = exc[[a for a in exc.columns 
+                    if 'Latitude/ N' in exc[a].to_numpy()][0]+1]
+        lon_deg = exc[[a for a in exc.columns 
+                    if 'Longitude/ E' in exc[a].to_numpy()][0]]
+        lon_min = exc[[a for a in exc.columns 
+                    if 'Longitude/ E' in exc[a].to_numpy()][0]+1]
+        
+    # Determine if folder or file is given
+    if  '.mat' in files:
+        files = [files]
+    else:
+        files = glob.glob(files+'*.mat')
+        
+    out_data = {'CTD':{},'MIX':{},'DATA':{}}
+    for file in files:
+        st_name = int(file.split('.mat')[0][-4:])
+        raw_data = myloadmat(file)
+        data = {k:raw_data[k] for k in ['CTD','MIX','DATA']}
+        
+        for name in ['CTD','MIX']:
+            for var in ['LON','LAT','fname','date']:
+                data[name][var] = raw_data['STA'][var]
+            
+            if excel_file is not None:
+                try:
+                    index = np.where(st == st_name)[0][0]
+                    data[name]['LON'] = lon_deg[index] + float(lon_min[index])/60
+                    data[name]['LAT'] = lat_deg[index] + float(lat_min[index])/60
+                except:
+                    pass
+            try:
+                data[name]['z'] = gsw.z_from_p(data[name]['P'],data[name]['LAT']) 
+            except: # just use 60N as lat if lat is not provided
+                data[name]['z'] = gsw.z_from_p(data[name]['P'],60)
+                
+            # Something weird in the data...
+            data[name]['z'][np.isnan(data[name]['SIGTH'])] = np.nan
+            data[name]['BottomDepth'] = np.nanmax(-data[name]['z'])
+            data[name]['datetime'] = pd.to_datetime(data[name]['date']
+                                                    ,format='%d-%b-%Y %H:%M:%S')
+                                    
     
+        
+        
+        for name in ['CTD','MIX','DATA']:
+            out_data[name][st_name] = data[name]
+            
+    return out_data['CTD'],out_data['MIX'],out_data['DATA']
+
 def read_mooring_from_mat(matfile):
     '''
     Read mooring data prepared in a .mat file.
@@ -735,7 +882,7 @@ def read_thermosalinograph(path):
 ############################################################################
 def contour_section(X,Y,Z,Z2=None,ax=None,station_pos=None,cmap='jet',Z2_contours=None,
                     clabel='',bottom_depth=None,clevels=20,station_text='',
-                    interp_opt=1):
+                    interp_opt=1,tlocator=None):
     '''    
     Plots a filled contour plot of *Z*, with contourf of *Z2* on top to 
     the axes *ax*. It also displays the position of stations, if given in
@@ -773,7 +920,10 @@ def contour_section(X,Y,Z,Z2=None,ax=None,station_pos=None,cmap='jet',Z2_contour
         instance. The default is ''.
     interp_opt: int, optional
         Indicator which is used to decide whether to use pcolormesh or contourf 
-
+    tlocator: matplotlib.ticker locators, optional
+        special locator for the colorbar. For example logarithmic values, 
+        for that use matplotlib.ticker.LogLocator(). Default is None.
+        
     Returns
     -------
     ax : plot axes
@@ -796,18 +946,27 @@ def contour_section(X,Y,Z,Z2=None,ax=None,station_pos=None,cmap='jet',Z2_contour
         y_limits = (0,np.nanmax(bottom_depth))
      
     if interp_opt == 0: #only z-interpolation: use pcolormesh
+        norm = None
         if type(clevels) == int:
+            if tlocator == 'logarithmic':
+                norm = matplotlib.colors.LogNorm(np.nanmin(Z),np.nanmax(Z))
             cmap = plt.cm.get_cmap(cmap,clevels)
-            cT = ax.pcolormesh(X,Y,Z,cmap=cmap,shading='auto') # draw Z
         else:
+            norm = matplotlib.colors.BoundaryNorm(clevels,
+                                                  ncolors=len(clevels)-1, 
+                                                  clip=False)
+            if tlocator is 'logarithmic':
+                norm = matplotlib.colors.LogNorm(np.min(clevels),np.max(clevels))
             cmap = plt.cm.get_cmap(cmap,len(clevels))
-            cT = ax.pcolormesh(X,Y,Z,cmap=cmap,shading='auto',
-                           norm = matplotlib.colors.BoundaryNorm(clevels, 
-                                                          ncolors=len(clevels)-1, 
-                                                          clip=False)) # draw Z
+            
+        cT = ax.pcolormesh(X,Y,Z,cmap=cmap,shading='auto',norm=norm) # draw Z
         plt.xlim(np.nanmin(X),np.nanmax(X))
     else: # full interpolation: use contours
-        cT = ax.contourf(X,Y,Z,cmap=cmap,levels=clevels,extend='both') # draw Z
+        locator = None
+        if tlocator == 'logarithmic':
+            locator = matplotlib.ticker.LogLocator()
+        cT = ax.contourf(X,Y,Z,cmap=cmap,levels=clevels,extend='both',
+                         locator=locator) # draw Z
     
     
     if Z2 is not None:
@@ -845,7 +1004,7 @@ def contour_section(X,Y,Z,Z2=None,ax=None,station_pos=None,cmap='jet',Z2_contour
     return ax, cT, cSIG
     
 def plot_CTD_section(CTD,stations,section_name='',cruise_name = '',
-                     x_type='distance',interp_opt = 1):
+                     x_type='distance',interp_opt = 1,z_fine=False):
     '''
     This function plots a CTD section of Temperature and Salinity,
     given CTD data either directly or via a file.
@@ -868,6 +1027,8 @@ def plot_CTD_section(CTD,stations,section_name='',cruise_name = '',
                      0: no interpolation,
                      1: linear interpolation, fine grid (default),
                      2: linear interpolation, coarse grid. The default is 1.
+    z_fine: Whether to use a fine z grid. If True, will be 10 cm, otherwise 1 m
+    
     Returns
     -------
     axT: matplotlib.pyplot.axes
@@ -907,7 +1068,7 @@ def plot_CTD_section(CTD,stations,section_name='',cruise_name = '',
     # put the fields (the vector data) on a regular, common pressure and X grid
     # by interpolating. 
     fCTD,Z,X,station_locs = CTD_to_grid(CTD,x_type=x_type,
-                                        interp_opt=interp_opt)
+                                        interp_opt=interp_opt,z_fine=z_fine)
     
     # plot the figure
     fig,[axT,axS] = plt.subplots(2,1,figsize=(8,9))
@@ -940,7 +1101,8 @@ def plot_CTD_section(CTD,stations,section_name='',cruise_name = '',
     
 def plot_CTD_single_section(CTD,stations,section_name='',cruise_name = '',
                      x_type='distance',parameter='T',clabel='Temperature [˚C]',
-                     cmap=cmocean.cm.thermal,clevels=20,interp_opt = 1):
+                     cmap=cmocean.cm.thermal,clevels=20,interp_opt = 1,
+                     tlocator=None,z_fine=False):
     '''
     This function plots a CTD section of a chosen variable,
     given CTD data either directly (through `CTD`) or via a file (through)
@@ -974,6 +1136,11 @@ def plot_CTD_single_section(CTD,stations,section_name='',cruise_name = '',
                      0: no interpolation,
                      1: linear interpolation, fine grid (default),
                      2: linear interpolation, coarse grid. The default is 1.
+    tlocator: matplotlib.ticker locators, optional
+        special locator for the colorbar. For example logarithmic values, 
+        for that use matplotlib.ticker.LogLocator(). Default is None.
+    z_fine: Whether to use a fine z grid. If True, will be 10 cm, otherwise 1 m
+        
     Returns
     -------
     None.
@@ -1006,7 +1173,7 @@ def plot_CTD_single_section(CTD,stations,section_name='',cruise_name = '',
     # put the fields (the vector data) on a regular, common pressure and X grid
     # by interpolating. 
     fCTD,Z,X,station_locs = CTD_to_grid(CTD,x_type=x_type,
-                                        interp_opt=interp_opt)
+                                        interp_opt=interp_opt,z_fine=z_fine)
     
     # plot the figure
     fig,ax = plt.subplots(1,1,figsize=(8,5))
@@ -1016,7 +1183,7 @@ def plot_CTD_single_section(CTD,stations,section_name='',cruise_name = '',
                           station_pos=station_locs,cmap=cmap,
                           clabel=clabel,bottom_depth=BDEPTH,
                           station_text=section_name,clevels=clevels,
-                          interp_opt=interp_opt)
+                          interp_opt=interp_opt,tlocator=tlocator)
     # Add x and y labels
     ax.set_ylabel('Depth [m]')
     if x_type == 'distance':
@@ -1397,7 +1564,7 @@ def create_empty_ts(T_extent,S_extent,p_ref = 0):
     
 def plot_ADCP_CTD_section(ADCP,CTD,stations,levels=np.linspace(-0.1,0.1,11),
                           geostr=False,levels_2 = np.linspace(-0.5,0.5,11),
-                          topography = None):
+                          topography = None,z_fine=False):
     '''
     
     Plots ADCP velocities along a CTD section given by *stations*. If wished, 
@@ -1421,6 +1588,8 @@ def plot_ADCP_CTD_section(ADCP,CTD,stations,levels=np.linspace(-0.1,0.1,11),
     topography : str or array-like, optional
         The topography to use for the map of the section. See documentation of
         Ocean.plot_CTD_map() . Default is None.
+    z_fine: Whether to use a fine z grid. If True, will be 10 cm, otherwise 1 m
+    
     Returns
     -------
     None.
@@ -1483,7 +1652,8 @@ def plot_ADCP_CTD_section(ADCP,CTD,stations,levels=np.linspace(-0.1,0.1,11),
     if geostr:
         # put the fields (the vector data) on a regular, common pressure and X grid
         # by interpolating. 
-        fCTD,Z,X,station_locs = CTD_to_grid(CTD,stations=stations,interp_opt=0)
+        fCTD,Z,X,station_locs = CTD_to_grid(CTD,stations=stations,interp_opt=0,
+                                            z_fine=z_fine)
 
         lat = [CTD[k]['LAT'] for k in stations]
         lon = [CTD[k]['LON'] for k in stations]
